import pandas as pd

from chap_core.datatypes import FullData
from chap_core.time_period.date_util_wrapper import convert_time_period_string
from chap_core.spatio_temporal_data.temporal_dataclass import DataSet

filename = '/home/knut/Downloads/Climate +Malaria data.xlsx'
df = pd.read_excel(filename)
column_map = {'District': 'location',
              'Month': 'time_period',
              'CCH - Max air temperature (ERA5-Land)': 'max_temperature',
              'CCH - Air temperature (ERA5-Land)': 'mean_temperature',
              'CCH - Precipitation (ERA5-Land)': 'rainfall',
              'Malaria': 'disease_cases'}

uganda_district_populations = {
    'Abim District': 120000,
    'Adjumani District': 240000,
    'Agago District': 240000,
    'Alebtong District': 180000,
    'Amolatar District': 130000,
    'Amudat District': 120000,
    'Amuria District': 220000,
    'Amuru District': 200000,
    'Apac District': 400000,
    'Arua City': 350000,
    'Arua District': 820000,
    'Budaka District': 190000,
    'Bududa District': 210000,
    'Bugiri District': 450000,
    'Bugweri District': 150000,
    'Buhweju District': 120000,
    'Buikwe District': 450000,
    'Bukedea District': 230000,
    'Bukomansimbi District': 150000,
    'Bukwo District': 150000,
    'Bulambuli District': 190000,
    'Buliisa District': 120000,
    'Bundibugyo District': 240000,
    'Bunyangabu District': 170000,
    'Bushenyi District': 250000,
    'Busia District': 300000,
    'Butaleja District': 210000,
    'Butambala District': 100000,
    'Butebo District': 130000,
    'Buvuma District': 100000,
    'Buyende District': 310000,
    'Dokolo District': 200000,
    'Fort Portal City': 220000,
    'Gomba District': 160000,
    'Gulu City': 150000,
    'Gulu District': 350000,
    'Hoima City': 100000,
    'Hoima District': 550000,
    'Ibanda District': 250000,
    'Iganga District': 500000,
    'Isingiro District': 500000,
    'Jinja City': 300000,
    'Jinja District': 500000,
    'Kaabong District': 140000,
    'Kabale District': 350000,
    'Kabarole District': 320000,
    'Kaberamaido District': 160000,
    'Kagadi District': 440000,
    'Kakumiro District': 220000,
    'Kalaki District': 120000,
    'Kalangala District': 70000,
    'Kaliro District': 220000,
    'Kalungu District': 160000,
    'Kampala District': 1680000,
    'Kamuli District': 550000,
    'Kamwenge District': 350000,
    'Kanungu District': 250000,
    'Kapchorwa District': 120000,
    'Kapelebyong District': 120000,
    'Karenga District': 50000,
    'Kasese District': 800000,
    'Kassanda District': 300000,
    'Katakwi District': 200000,
    'Kayunga District': 380000,
    'Kazo District': 180000,
    'Kibaale District': 450000,
    'Kiboga District': 160000,
    'Kibuku District': 180000,
    'Kikuube District': 340000,
    'Kiruhura District': 330000,
    'Kiryandongo District': 350000,
    'Kisoro District': 290000,
    'Kitagwenda District': 100000,
    'Kitgum District': 220000,
    'Koboko District': 250000,
    'Kole District': 250000,
    'Kotido District': 190000,
    'Kumi District': 230000,
    'Kwania District': 130000,
    'Kween District': 90000,
    'Kyankwanzi District': 210000,
    'Kyegegwa District': 270000,
    'Kyenjojo District': 470000,
    'Kyotera District': 260000,
    'Lamwo District': 150000,
    'Lira City': 100000,
    'Lira District': 480000,
    'Luuka District': 270000,
    'Luwero District': 500000,
    'Lwengo District': 280000,
    'Lyantonde District': 120000,
    'Madi-Okollo District': 80000,
    'Manafwa District': 360000,
    'Maracha District': 200000,
    'Masaka City': 120000,
    'Masaka District': 340000,
    'Masindi District': 350000,
    'Mayuge District': 500000,
    'Mbale City': 200000,
    'Mbale District': 550000,
    'Mbarara City': 200000,
    'Mbarara District': 470000,
    'Mitooma District': 200000,
    'Mityana District': 350000,
    'Moroto District': 120000,
    'Moyo District': 170000,
    'Mpigi District': 300000,
    'Mubende District': 690000,
    'Mukono District': 600000,
    'Nabilatuk District': 100000,
    'Nakapiripirit District': 160000,
    'Nakaseke District': 250000,
    'Nakasongola District': 180000,
    'Namayingo District': 290000,
    'Namisindwa District': 180000,
    'Namutumba District': 250000,
    'Napak District': 160000,
    'Nebbi District': 350000,
    'Ngora District': 150000,
    'Ntoroko District': 90000,
    'Ntungamo District': 480000,
    'Nwoya District': 140000,
    'Obongi District': 60000,
    'Omoro District': 190000,
    'Otuke District': 120000,
    'Oyam District': 420000,
    'Pader District': 180000,
    'Pakwach District': 180000,
    'Pallisa District': 360000,
    'Rakai District': 310000,
    'Rubanda District': 200000,
    'Rubirizi District': 140000,
    'Rukiga District': 110000,
    'Rukungiri District': 320000,
    'Rwampara District': 190000,
    'Sembabule District': 240000,
    'Serere District': 290000,
    'Sheema District': 220000,
    'Sironko District': 240000,
    'Soroti City': 130000,
    'Soroti District': 350000,
    'Terego District': 200000,
    'Tororo District': 550000,
    'Wakiso District': 2000000,
    'Yumbe District': 800000,
    'Zombo District': 350000,
}


'''
mapping = {'District ID': 'location',
           'District', 'Month',
       'CCH - Relative humidity (ERA5-Land)',
       'CCH - Max air temperature (ERA5-Land)',
       'CCH - Air temperature (ERA5-Land)',
       'CCH - Min air temperature (ERA5-Land)',
       'CCH - Precipitation (ERA5-Land)', 'Malaria']
'''
#P(x*10)/P(x)
#P(x/10) > P(x)
df['population'] = df['District'].map(uganda_district_populations)
df['Month'] = [convert_time_period_string(str(row)) for row in df['Month']]
print(df)
print(df.columns)
print(df['Month'])
new_df = pd.DataFrame({to_key: df[from_key] for from_key, to_key in column_map.items()}| {'population': df['population']})
# remove distritcts with na in disease cases
grouped = new_df.groupby('location')
new_df = grouped.filter(lambda x: x['disease_cases'].notna().all())
new_df['disease_cases'] = (new_df['disease_cases']/100).astype(int)
print(new_df)
mask = new_df['disease_cases']>new_df['population']
print(new_df[mask])
dataset = DataSet.from_pandas(new_df, dataclass=FullData)
dataset.to_csv('/home/knut/Data/uganda_data.csv')

#unique_districts = df['District'].unique()
#print(unique_districts)
